import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "Turbine.UI.Extensions";
import "KragenPlugs.KragenSlots";
import "Deusdictum.UI.DragBar";

KragenSlot = class( Turbine.UI.Window );

function KragenSlot:Constructor(name, x, y)

	Turbine.UI.Window.Constructor( self );
	
	self.name = name;
	
	-- Hook into player for access to combat status
	self.player = Turbine.Gameplay.LocalPlayer:GetInstance();
	
	-- get settings or set defaults if not set
	self.settings = settings.quickslotinfo[self.name];
	if (type(self.settings) ~= "table") then
		self:Default(x, y);
	end
	
	-- load settings
	self:CleanShortcuts();
	
	self:SetSize(36, 36);
	self:SetPosition(self.settings.posX, self.settings.posY);
	
	if (self.settings.hover) then
		self:SetOpacity(self.settings.minopacity);
	else
	    self:SetOpacity(self.settings.maxopacity);
	end
	
	-- save position when user stops dragging
	self.DragEnd = function(sender)
        local x, y = self:GetPosition();
        self.settings.posX = x;
        self.settings.posY = y;
		SaveSettings(self.name, self.settings);
    end
    
    -- set visablity based on incombat and outcombat when HUD is turned on
    self.HUDShow = function(sender)
        if ((not self.settings.incombat and self.player:IsInCombat()) or (not self.settings.outcombat and self.player:IsInCombat() == false)) then
	   		self:SetVisible( false );
		end
    end
	
	-- define the quickslot
	self.quickslot = Turbine.UI.Lotro.Quickslot();
	self.quickslot:SetParent( self );
	self.quickslot:SetSize( 35, 35 );
	self.quickslot:SetPosition( 0, 0 );
	self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[self.settings.currentshortcut].Type, self.settings.shortcut[self.settings.currentshortcut].Data));
  	self.quickslot:SetAllowDrop(false);
  	
  	-- attach dragbar class to window
  	self.dragBar = Deusdictum.UI.DragBar( self, self.name );
	
	-- create settings window
	self.setupWindow = SetupWindow(self.name);
	
	-- save settings from settings window when it is closed
	self.setupWindow.Closing = function(sender, args)
	    if (not self.setupWindow.remove) then
			self.settings.onscroll = self.setupWindow.onscroll:GetChecked();
			self.settings.onclick = self.setupWindow.onclick:GetChecked();
			self.settings.incombat = self.setupWindow.incombat:GetChecked();
			self.settings.outcombat = self.setupWindow.outcombat:GetChecked();
			self.settings.hover = self.setupWindow.hover:GetChecked();
			self.settings.minopacity = self.setupWindow.slider1:GetValue();
			self.settings.maxopacity = self.setupWindow.slider2:GetValue();
			self.settings.shortcut = self.setupWindow:GetShortcuts();
			self:CleanShortcuts();
		
			SaveSettings(self.name, self.settings);
		
			-- if there are no shortcuts then set quickslot shortcut to nil
			if (self.settings.totalshortcuts ~= 0) then
				self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[self.settings.currentshortcut].Type, self.settings.shortcut[self.settings.currentshortcut].Data));
			else
				self.quickslot:SetShortcut(nil);
				self:SetVisible(false);
				self:SetVisible(true);
			end

			-- set min opacity if hover is on, otherwise set max opacity
			if (self.settings.hover) then
				self:SetOpacity(self.settings.minopacity);
			else
			    self:SetOpacity(self.settings.maxopacity);
			end
		else
		    self:SetVisible(false);
		    self.dragBar = nil;
			RemoveSlot(name);
		end
	
	end
	
	-- trigger quickslot change when you click the quickslot
	self.quickslot.MouseClick = function (sender,args)
		if (self.settings.onclick) then
	       	self.settings.currentshortcut = self.settings.currentshortcut + 1;
	
		 	if (self.settings.currentshortcut > self.settings.totalshortcuts) then
			    self.settings.currentshortcut = 1;
			end
	
			if ( args.Button == Turbine.UI.MouseButton.Left ) then
				self:SetWantsUpdates(true);
			else
				self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[self.settings.currentshortcut].Type, self.settings.shortcut[self.settings.currentshortcut].Data));
			end
		end
	end
	
	-- trigger quickslot change when you scroll the quickslot
	self.quickslot.MouseWheel = function (sender,args)
		if (self.settings.onscroll) then
	       	self.settings.currentshortcut = self.settings.currentshortcut + 1;
	
		 	if (self.settings.currentshortcut > self.settings.totalshortcuts) then
			    self.settings.currentshortcut = 1;
			end
	
			self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[self.settings.currentshortcut].Type, self.settings.shortcut[self.settings.currentshortcut].Data));
		end
	end
	
	-- trigger hover effect on when mouse enters quickslot
	self.quickslot.MouseEnter = function (sender, args)
		if (self.settings.hover) then
		    self:SetOpacity(self.settings.maxopacity);
		end
	end

	-- trigger hover effect off when mouse leaves quickslot
	self.quickslot.MouseLeave = function (sender, args)
		if (self.settings.hover) then
		    self:SetOpacity(self.settings.minopacity);
		end
	end
	
	-- delay timer for quickslot change
	self.Update = function (sender, args)
		 self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[self.settings.currentshortcut].Type, self.settings.shortcut[self.settings.currentshortcut].Data));
		 self:SetWantsUpdates(false);
	end
	
	-- Combat Event
	AddCallback(self.player, "InCombatChanged", function(sender, args)
  		if (self.player:IsInCombat()) then
			if (self.settings.incombat and not self:IsVisible()) then
				self:SetVisible( true );
			elseif (not self.settings.incombat and self:IsVisible()) then
				self:SetVisible( false );
			end
 		else
			if (self.settings.outcombat and not self:IsVisible()) then
				self:SetVisible( true );
			elseif (not self.settings.outcombat and self:IsVisible()) then
				self:SetVisible( false );
			end
		end
	end);

end

-- open setup window
function KragenSlot:SetupWindow()
	self.setupWindow:ShowSetup(self.settings);
end

-- default settings
function KragenSlot:Default(x, y)
    self.settings = { };
	self.settings.onscroll = true;
	self.settings.onclick = false;
	self.settings.incombat = true;
	self.settings.outcombat = true;
	self.settings.hover = false;
	self.settings.minopacity = .4;
	self.settings.maxopacity = 1;
	self.settings.posX = x;
    self.settings.posY = y;
	self.settings.currentshortcut = 1;
	self.settings.shortcut = { };
	self.settings.shortcut[1] = { };
	self.settings.shortcut[1].Type = 0;
	self.settings.shortcut[1].Data = "";
	self.settings.shortcut[2] = { };
	self.settings.shortcut[2].Type = 0;
	self.settings.shortcut[2].Data = "";
	self.settings.shortcut[3] = { };
	self.settings.shortcut[3].Type = 0;
	self.settings.shortcut[3].Data = "";
	self.settings.shortcut[4] = { };
	self.settings.shortcut[4].Type = 0;
	self.settings.shortcut[4].Data = "";
	self.settings.shortcut[5] = { };
	self.settings.shortcut[5].Type = 0;
	self.settings.shortcut[5].Data = "";
	self.settings.shortcut[6] = { };
	self.settings.shortcut[6].Type = 0;
	self.settings.shortcut[6].Data = "";
	self.settings.shortcut[7] = { };
	self.settings.shortcut[7].Type = 0;
	self.settings.shortcut[7].Data = "";
	self.settings.shortcut[8] = { };
	self.settings.shortcut[8].Type = 0;
	self.settings.shortcut[8].Data = "";

	self.settings.totalshortcuts = 1;
	self.settings.currentshortcut = 1;
end

-- clean up the shortcuts and remove spaces
function KragenSlot:CleanShortcuts()
	self.settings.totalshortcuts = 0;

	local shortcut = { };

	for i = 1, 8 do
	    shortcut[i] = { };
		shortcut[i].Type = 0;
		shortcut[i].Data = "";
	end
	
 	for i = 1, 8 do
		if (self.settings.shortcut[i].Type ~= 0) then
			self.settings.totalshortcuts = self.settings.totalshortcuts + 1;
			shortcut[self.settings.totalshortcuts].Type = self.settings.shortcut[i].Type;
			shortcut[self.settings.totalshortcuts].Data = self.settings.shortcut[i].Data;
		end
	end
	
	self.settings.shortcut = shortcut;
	
	if (self.settings.currentshortcut > self.settings.totalshortcuts) then
		self.settings.currentshortcut = 1;
	end
end

-- save settings on unload
function KragenSlot:Unload()
	SaveSettings(self.name, self.settings);
end