import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.UI.Extensions";
import "Turbine.Gameplay";
import "KragenPlugs.DUInterface";
import "KragenPlugs.KragenSlots";

-- screen width and height for default quickslot position
local displayWidth = Turbine.UI.Display:GetWidth();
local displayHeight = Turbine.UI.Display:GetHeight();

-- table for holding instances of KragenSlot
slots = { };

-- load and iniate saved KragenSlots
function LoadSettings()
	settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "KragenSlotsSettings" );
	if (settings == nil) then
		settings = { };
		settings.quickslotinfo = { };
		settings.slots = { };
		settings.totalslots = 0;
		Turbine.Shell.WriteLine("Welcome to KragenSlots. \n To create a new quickslot, or to pull up the setup dialog for an existing quickslot type /kragenslots <unique name> \n\n\n To see a list of existing quickslots type /kragenslots");
	else
	    for i = 1, settings.totalslots do
			slots[i] = KragenSlot(settings.slots[i]);
		end
	end
end

-- function to save a quickslots info (called from KragenSlot class)
function SaveSettings(name, value)
	settings.quickslotinfo[name] = value;
	Turbine.PluginData.Save( Turbine.DataScope.Character, "KragenSlotsSettings", settings );
end

-- on unload call each instances unload function to save settings
function UnloadPlugin()
	for i = 1, settings.totalslots do
		slots[i]:Unload();
	end
end

function AddCallback(object, event, callback)
        if (object[event] == nil) then
                object[event] = callback;
        else
                if (type(object[event]) == "table") then
                        table.insert(object[event], callback);
                else
                        object[event] = {object[event], callback};
                end
        end
end


-- function to remove quickslot from settings file and from the screen
function RemoveSlot(name)
	local tempslots = { };
	local temptotal = 0;
	settings.quickslotinfo[name] = nil;
	for i = 1, settings.totalslots do
		if (settings.slots[i] == name) then
			settings.slots[i] = nil;
			slots[i] = nil;
		else
			temptotal = temptotal + 1;
		    tempslots[temptotal] = settings.slots[i];
		    slots[temptotal] = slots[i];
		end
	end
	
	settings.totalslots = temptotal;
	settings.slots = tempslots;
	
    Turbine.PluginData.Save( Turbine.DataScope.Character, "KragenSlotsSettings", settings );
end


-- setup chat commands
Command = Turbine.ShellCommand();

function Command:Execute( command, arguments )
	local new = true;
	
	for i = 1, settings.totalslots do
		if (arguments == settings.slots[i]) then
			slots[i]:SetupWindow();
			new = false;
		end
	end
	
	if (arguments == "" or arguments == " ") then
	    Turbine.Shell.WriteLine("Existing slot names: ");
     	for i = 1, settings.totalslots do
			Turbine.Shell.WriteLine("      " .. settings.slots[i] .. " ");
		end
		
	elseif (arguments == "help") then
	    Turbine.Shell.WriteLine("To create a new quickslot, or to pull up the setup dialog for an existing quickslot type /kragenslots <unique name> \n\n\n To see a list of existing quickslots type /kragenslots");
	
	elseif (new) then
	    settings.totalslots = settings.totalslots + 1;
		slots[settings.totalslots] = KragenSlot(arguments, settings.totalslots * 36 + (displayWidth * 3 / 4), displayHeight * 1 / 3);
		settings.slots[settings.totalslots] = arguments;
		slots[settings.totalslots]:SetupWindow();
	end

end

-- inialize plugin
function init()
	-- setup plugin unload function to save setting on unload.
	pluginName="KragenSlots";
	window = Turbine.UI.Window();
	window:SetWantsUpdates(true);
	window.Update = function(sender,args)
		if (Plugins[pluginName] ~= nil) then
			Plugins[pluginName].Unload = function(self,sender,args)
				UnloadPlugin();
			end
			window:SetWantsUpdates(false);
		end
	end
	
	-- add chat commands to turbine shell
	Turbine.Shell.AddCommand( "kragenslots", Command );
	
	-- load settings and inialize saved KragenSlots
	LoadSettings();
end


init();